"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MidwayKafkaFramework = void 0;
const core_1 = require("@midwayjs/core");
const kafka_1 = require("./kafka");
const decorator_1 = require("./decorator");
const kafkajs_1 = require("kafkajs");
const manager_1 = require("./manager");
const toMidwayLogLevel = level => {
    switch (level) {
        case kafkajs_1.logLevel.NOTHING:
            return 'none';
        case kafkajs_1.logLevel.ERROR:
            return 'error';
        case kafkajs_1.logLevel.WARN:
            return 'warn';
        case kafkajs_1.logLevel.INFO:
            return 'info';
        case kafkajs_1.logLevel.DEBUG:
            return 'debug';
    }
};
let MidwayKafkaFramework = class MidwayKafkaFramework extends core_1.BaseFramework {
    configure() {
        return this.configService.getConfiguration('kafka');
    }
    async applicationInitialize() {
        this.LogCreator = logLevel => {
            const logger = this.kafKaLogger;
            return ({ level, log }) => {
                const lvl = toMidwayLogLevel(level);
                const { message, ...extra } = log;
                logger === null || logger === void 0 ? void 0 : logger[lvl](message, extra);
            };
        };
        // Create a connection manager
        if (this.configurationOptions['kafkaConfig']) {
            this.app = new kafka_1.KafkaConsumerServer({
                logger: this.kafKaLogger,
                ...this.configurationOptions,
            });
        }
        else {
            this.app = {};
        }
    }
    async run() {
        if (this.configurationOptions['kafkaConfig']) {
            try {
                await this.app.connect(this.configurationOptions.kafkaConfig, this.configurationOptions.consumerConfig);
                await this.loadLegacySubscriber();
                this.kafKaLogger.info('Kafka consumer server start success');
            }
            catch (error) {
                this.kafKaLogger.error('Kafka consumer connect fail', error);
                await this.app.closeConnection();
            }
        }
        else {
            const { consumer } = this.configurationOptions;
            if (!consumer)
                return;
            const subscriberMap = {};
            // find subscriber
            const subscriberModules = (0, core_1.listModule)(decorator_1.KAFKA_DECORATOR_KEY);
            for (const subscriberModule of subscriberModules) {
                const subscriberName = (0, core_1.getClassMetadata)(decorator_1.KAFKA_DECORATOR_KEY, subscriberModule);
                subscriberMap[subscriberName] = subscriberModule;
            }
            this.typedResourceManager = new core_1.TypedResourceManager({
                initializeValue: consumer,
                initializeClzProvider: subscriberMap,
                resourceInitialize: async (resourceInitializeConfig, resourceName) => {
                    let client;
                    if (resourceInitializeConfig.kafkaInstanceRef) {
                        client = manager_1.KafkaManager.getInstance().getKafkaInstance(resourceInitializeConfig.kafkaInstanceRef);
                        if (!client) {
                            throw new core_1.MidwayCommonError(`kafka instance ${resourceInitializeConfig.kafkaInstanceRef} not found`);
                        }
                    }
                    else {
                        client = new kafkajs_1.Kafka({
                            logCreator: this.LogCreator,
                            ...resourceInitializeConfig.connectionOptions,
                        });
                        manager_1.KafkaManager.getInstance().addKafkaInstance(resourceName, client);
                    }
                    const consumer = client.consumer(resourceInitializeConfig.consumerOptions);
                    await consumer.connect();
                    await consumer.subscribe(resourceInitializeConfig.subscribeOptions);
                    return consumer;
                },
                resourceBinding: async (ClzProvider, resourceInitializeConfig, consumer) => {
                    const runMethod = ClzProvider.prototype['eachBatch']
                        ? 'eachBatch'
                        : 'eachMessage';
                    const runConfig = {
                        ...resourceInitializeConfig.consumerRunConfig,
                    };
                    runConfig[runMethod] = async (payload) => {
                        const ctx = this.app.createAnonymousContext();
                        const fn = await this.applyMiddleware(async (ctx) => {
                            ctx.payload = payload;
                            ctx.consumer = consumer;
                            const instance = await ctx.requestContext.getAsync(ClzProvider);
                            return await instance[runMethod].call(instance, payload, ctx);
                        });
                        return await fn(ctx);
                    };
                    return runConfig;
                },
                resourceStart: async (resource, resourceInitializeConfig, resourceBindingResult) => {
                    await resource.run(resourceBindingResult);
                },
                resourceDestroy: async (resource) => {
                    await resource.disconnect();
                },
            });
            await this.typedResourceManager.init();
            await this.typedResourceManager.start();
        }
    }
    async loadLegacySubscriber() {
        const subscriberModules = (0, core_1.listModule)(core_1.MS_CONSUMER_KEY, module => {
            const metadata = (0, core_1.getClassMetadata)(core_1.MS_CONSUMER_KEY, module);
            return metadata.type === core_1.MSListenerType.KAFKA;
        });
        for (const module of subscriberModules) {
            const data = (0, core_1.listPropertyDataFromClass)(core_1.MS_CONSUMER_KEY, module);
            const topicTitles = [...new Set(data.map(e => e[0].topic))];
            const midwayConsumerConfigs = topicTitles.map(e => {
                const midwayConsumerConfig = {
                    topic: e,
                    subscription: {},
                    runConfig: {},
                };
                const consumerParams = data
                    .map(value => {
                    if (value[0].topic === midwayConsumerConfig.topic) {
                        return Object.assign(midwayConsumerConfig, value[0]);
                    }
                })
                    .filter(e => e && true);
                if (consumerParams && Object.keys(consumerParams[0]).length > 0) {
                    return consumerParams[0];
                }
                return midwayConsumerConfig;
            });
            midwayConsumerConfigs.forEach(async (e) => {
                await this.app.connection.subscribe(Object.assign({
                    topics: topicTitles,
                }, e.subscription));
                await this.app.connection.run(Object.assign(e.runConfig, {
                    eachMessage: async (payload) => {
                        const { topic, partition, message } = payload;
                        let propertyKey;
                        for (const methodBindListeners of data) {
                            for (const listenerOptions of methodBindListeners) {
                                if (topic === listenerOptions.topic) {
                                    propertyKey = listenerOptions.propertyKey;
                                    const ctx = {
                                        topic: topic,
                                        partition: partition,
                                        message: message,
                                        commitOffsets: (offset) => {
                                            return this.app.connection.commitOffsets([
                                                {
                                                    topic: topic,
                                                    partition: partition,
                                                    offset,
                                                },
                                            ]);
                                        },
                                        payload,
                                        consumer: this.app.connection,
                                    };
                                    this.app.createAnonymousContext(ctx);
                                    if (typeof propertyKey === 'string') {
                                        const isPassed = await this.app
                                            .getFramework()
                                            .runGuard(ctx, module, propertyKey);
                                        if (!isPassed) {
                                            throw new core_1.MidwayInvokeForbiddenError(propertyKey, module);
                                        }
                                    }
                                    const ins = await ctx.requestContext.getAsync(module);
                                    const fn = await this.applyMiddleware(async () => {
                                        return await ins[propertyKey].call(ins, message);
                                    });
                                    try {
                                        const result = await fn(ctx);
                                        // 返回为undefined，下面永远不会执行
                                        if (result) {
                                            const res = await this.app.connection.commitOffsets([
                                                {
                                                    topic: topic,
                                                    partition: partition,
                                                    offset: message.offset,
                                                },
                                            ]);
                                            return res;
                                        }
                                    }
                                    catch (error) {
                                        // 记录偏移量提交的异常情况
                                        this.logger.error(error);
                                    }
                                }
                            }
                        }
                    },
                }));
            });
        }
    }
    getConsumer(subscriberNameOrInstanceName) {
        if (this.typedResourceManager) {
            return this.typedResourceManager.getResource(subscriberNameOrInstanceName);
        }
    }
    getKafka(instanceName) {
        return manager_1.KafkaManager.getInstance().getKafkaInstance(instanceName);
    }
    getFrameworkName() {
        return 'midway:kafka';
    }
    async beforeStop() {
        if (this.typedResourceManager) {
            await this.typedResourceManager.destroy();
        }
    }
    getFrameworkType() {
        return core_1.MidwayFrameworkType.MS_KAFKA;
    }
};
__decorate([
    (0, core_1.Logger)('kafkaLogger'),
    __metadata("design:type", Object)
], MidwayKafkaFramework.prototype, "kafKaLogger", void 0);
MidwayKafkaFramework = __decorate([
    (0, core_1.Framework)()
], MidwayKafkaFramework);
exports.MidwayKafkaFramework = MidwayKafkaFramework;
//# sourceMappingURL=framework.js.map