"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.KafkaAdminFactory = exports.KafkaProducerFactory = void 0;
const core_1 = require("@midwayjs/core");
const kafkajs_1 = require("kafkajs");
const manager_1 = require("./manager");
let KafkaProducerFactory = class KafkaProducerFactory extends core_1.ServiceFactory {
    getName() {
        return 'kafka:producer';
    }
    async init() {
        await this.initClients(this.pubConfig);
    }
    async createClient(config, clientName) {
        const { connectionOptions, producerOptions, kafkaInstanceRef } = config;
        let client;
        if (kafkaInstanceRef) {
            client = manager_1.KafkaManager.getInstance().getKafkaInstance(kafkaInstanceRef);
            if (!client) {
                throw new core_1.MidwayCommonError(`[midway:kafka] kafka instance ${kafkaInstanceRef} not found`);
            }
        }
        else {
            client = new kafkajs_1.Kafka(connectionOptions);
            manager_1.KafkaManager.getInstance().addKafkaInstance(kafkaInstanceRef, client);
        }
        const producer = client.producer(producerOptions);
        producer.on('producer.connect', () => {
            this.logger.info('[midway:kafka] producer: %s is connect', clientName);
        });
        await producer.connect();
        return producer;
    }
    async destroyClient(producer, name) {
        await producer.disconnect();
        this.logger.info('[midway:kafka] producer: %s is close', name);
    }
    async destroy() {
        await super.stop();
    }
};
__decorate([
    (0, core_1.Logger)('kafkaLogger'),
    __metadata("design:type", Object)
], KafkaProducerFactory.prototype, "logger", void 0);
__decorate([
    (0, core_1.Config)('kafka.producer'),
    __metadata("design:type", Object)
], KafkaProducerFactory.prototype, "pubConfig", void 0);
__decorate([
    (0, core_1.Init)(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], KafkaProducerFactory.prototype, "init", null);
__decorate([
    (0, core_1.Destroy)(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], KafkaProducerFactory.prototype, "destroy", null);
KafkaProducerFactory = __decorate([
    (0, core_1.Singleton)()
], KafkaProducerFactory);
exports.KafkaProducerFactory = KafkaProducerFactory;
let KafkaAdminFactory = class KafkaAdminFactory extends core_1.ServiceFactory {
    getName() {
        return 'kafka:admin';
    }
    async init() {
        await this.initClients(this.adminConfig);
    }
    async createClient(config, clientName) {
        const { connectionOptions, adminOptions, kafkaInstanceRef } = config;
        let client;
        if (kafkaInstanceRef) {
            client = manager_1.KafkaManager.getInstance().getKafkaInstance(kafkaInstanceRef);
            if (!client) {
                throw new core_1.MidwayCommonError(`[midway:kafka] kafka instance ${kafkaInstanceRef} not found`);
            }
        }
        else {
            client = new kafkajs_1.Kafka(connectionOptions);
            manager_1.KafkaManager.getInstance().addKafkaInstance(kafkaInstanceRef, client);
        }
        const admin = client.admin(adminOptions);
        await admin.connect();
        return admin;
    }
    async destroyClient(admin, name) {
        await admin.disconnect();
        this.logger.info('[midway:kafka] admin: %s is close', name);
    }
    async destroy() {
        await super.stop();
    }
};
__decorate([
    (0, core_1.Logger)('kafkaLogger'),
    __metadata("design:type", Object)
], KafkaAdminFactory.prototype, "logger", void 0);
__decorate([
    (0, core_1.Config)('kafka.admin'),
    __metadata("design:type", Object)
], KafkaAdminFactory.prototype, "adminConfig", void 0);
__decorate([
    (0, core_1.Init)(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], KafkaAdminFactory.prototype, "init", null);
__decorate([
    (0, core_1.Destroy)(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], KafkaAdminFactory.prototype, "destroy", null);
KafkaAdminFactory = __decorate([
    (0, core_1.Singleton)()
], KafkaAdminFactory);
exports.KafkaAdminFactory = KafkaAdminFactory;
//# sourceMappingURL=service.js.map